without warning
without trace
-----------------------------------------------------------------------------
-----------------------------------------------------------------------------
-- MATH
--/topic Math Routines
--/info
-- Miscellaneous mathematical and low-level routines

-----------------------------------------------------------------------------
--/topic Math Routines
--/func or_all( sequence s )
--/desc Performs /i or_bits on all the elements of the sequence.
--/ret ATOM: OR'd value of sequence elements.
--
-- Typically used to combine a series of attribute
-- flags together.
--
-- Example:
--
-- /code        
--      -- combine flags
--      integer flags
--      flags = or_all( { WS_CHILD, WS_VISIBLE, BS_PUSHBUTTON, WS_TABSTOP} )
-- /endcode


global function or_all(sequence s)
-- or together all elements of a sequence
    atom result
    
    result = 0
    for i = 1 to length(s) do
        result = or_bits(result, s[i])
    end for
    return result
end function

--/topic Math Routines
--/func get_bits( atom b32 )
--/desc Does the reverse of /or_all() in that it gets all the bit values from an atom.
--/ret Sequence: The non-zero bits set in /i b32.
--
-- Typically used to extract which bits have been set on in a flag.
--
-- Example:
--
-- /code        
--      -- combine flags
--      integer flags
--      sequence codes
--      codes = get_bits( origflag )
--      if find(WS_EX_CLIENTEDGE, codes) then
--          setText(statusarea, "Client Edge has been specified")
--      end if
-- /endcode
constant kBitPosn = {
    #00000001 , #00000002, #00000004, #00000008,
    #00000010 , #00000020, #00000040, #00000080,
    #00000100 , #00000200, #00000400, #00000800,
    #00001000 , #00002000, #00004000, #00008000,
    #00010000 , #00020000, #00040000, #00080000,
    #00100000 , #00200000, #00400000, #00800000,
    #01000000 , #02000000, #04000000, #08000000,
    #10000000 , #20000000, #40000000, #80000000
                     }
global function get_bits(atom b32)
sequence lResult

    lResult = {}
    for i = 1 to length(kBitPosn) do
        if and_bits(b32, kBitPosn[i]) then
            lResult &= kBitPosn[i]
        end if
    end for

    if length(lResult) = 0 then
        lResult = {0}
    end if
    
    return lResult
end function


-----------------------------------------------------------------------------
--/topic Math Routines
--/func lo_word( atom a )
--/desc Get low portion of number.
--/ret ATOM: Low 16 bits of a 32 bit word.
-- Typically used to decode Win32 message values when several values 
-- are packed into a single number. 
-- 
-- Example: 
--
-- /code
--      -- extract the low portion from lParam
--      integer x
--      x = lo_word( lParam )
-- /endcode

global function lo_word( atom a )
    -- return low word
    return and_bits( a, #FFFF )
end function

-----------------------------------------------------------------------------
--/topic Math Routines
--/func hi_word( atom a )
--/desc Returns high portion of word.
--/ret ATOM: High 16 bits of a 32 bit word.
-- Typically used to decode Win32 message values when several values 
-- are packed into a single number.
--
-- Example:
--
-- /code         
--      -- extract the high portion from lParam
--      integer y
--      y = hi_word( lParam )
-- /endcode

global function hi_word( atom a )
    -- return high word
    return floor(a / #10000 )
end function


-----------------------------------------------------------------------------
--/topic Math Routines
--/func unpack_word( atom a )
--/desc Returns high and low portions of a 32-bit word.
--/ret SEQUENCE: {High 16 bits,Low 16 bits}
-- Typically used to decode Win32 message values when several values 
-- are packed into a single number.
--
-- Example:
--
-- /code         
--      -- extract the short words from lParam
--      sequence hilo
--      hilo = upack_word( lParam )
-- /endcode

global function unpack_word( atom a )
    return {floor(a / #10000 ), and_bits(a, #FFFF)}
end function


-- NEW! 0.45f
-----------------------------------------------------------------------------
--/topic Math Routines
--/func pack_word( integer low, integer high )
--/desc Packs values into word.
--/ret ATOM: 32 bit word with low value in low 16 bits, high value in high 16 bits.
-- Typically used to encode a message parameter into a 32 bit word.
--
-- Example:
--
-- /code         
--      -- pack min and max into parameter
--      integer y
--      lParam = pack_word( min, max )
-- /endcode

global function pack_word( integer low, integer high )
    -- return packed 32 bit value
    
    -- mask values
    low  = and_bits( low,  #FFFF )
    high = and_bits( high, #FFFF )
    
    -- pack values
    return ( high * #10000 ) + low
    
end function


-----------------------------------------------------------------------------
--/topic Math Routines
--/func abs( object a )
--/desc Absolute value.
--/ret Absolute value of the atom or sequence.
--
--Example
--/code
--       sequence s
--       atom a
--
--       a = abs(-1)   -- = 1
--       s = abs({-2,-1,{-1,0,1},1,2}) -- = {2,1,{1,0,1},1,2}
--/endcode

global function abs( object a )
    if atom(a) then
        if a < 0 then
            return -a
        else
            return a
        end if
    else
        for i = 1 to length(a) do
            a[i] = abs(a[i])
        end for

        return a
    end if
    
end function


----------------------------------------------------------------------------
--/topic Math Routines
--/func shortInt( atom a )
--/desc Converts a number into a 16-bit signed integer
--/ret INTEGER: The value of /i a as a short int.
-- Typically used to decode Win32 message values
--
-- Example:
--
-- /code         
--      -- Mouse Position
--      sequence hilo, x, y
--      hilo = upack_word( lParam )
--      x = shortInt(hilo[2])
--      y = shortInt(hilo[1])
-- /endcode
global function shortInt( atom i )

    -- converts numbers ( 4 bytes #0000 to #FFFF)
    -- to signed short ints (2 bytes -32768 to 32767 )

    -- Force the use of only the rightmost 2 bytes.
    i = and_bits(i, #FFFF)

    if i >= 0 and i <= #7FFF then
        return i
    else
        return i - #10000
    end if
    
end function



--/topic Math Routines
--/func TextToNumber( sequence text )
--/ret Atom: The number represented by the text.
--/desc This converts the text into a number.
-- If the text contains invalid characters, zero is returned.
--
-- This function can optionally return information about invalid numbers. If
-- /i text has the form of {sequence, integer} then if the integer is nonzero,
-- a sequence is returned. The first element is the value converted, and the
-- second is the position in the text where conversion stopped. If no errors
-- were found then this is zero.
--
--/code
--     sequence rc
--     atom   val
--     rc = TextToNumber({"12.34a", 1})
--     --  rc ---> {12.34, 6}
--
--     val = TextToNumber("12.34a")
--     --  val ---> 0
--
--/endcode
global function TextToNumber( sequence text)
    -- get the numeric value of text
    integer dot,sign,tstart,tend, v, note, notify
    atom lhs, rhs, lh, rh

    dot = 0
    lh = 0
    lhs = 0
    rh = 0
    rhs = 1
    sign = 0
    note = 0

    if  length(text) = 2  and
        sequence(text[1]) and
        integer(text[2])
    then
        notify  = text[2]
        text = text[1]
    else
        notify = 0
    end if

    -- convert the value of the text    
    tstart = 1
    tend = length(text)
    -- Ignore leading whitespace
    while tstart <= tend do
        if equal(text[tstart], '-') and sign = 0 then
            sign = -1
        elsif equal (text[tstart],'+') and sign = 0 then
            sign = 1
        elsif find(text[tstart], {9, ' '}) = 0 then
            exit
        end if
        tstart += 1
    end while
    -- Ignore trailing whitespace
    while tstart <= tend do
        if equal(text[tend], '-') and sign = 0 then
            sign = -1
        elsif equal(text[tend],'+') and sign = 0 then
            sign = 1
        elsif find(text[tend], {9, ' '}) = 0 then
            exit
        end if
        tend -= 1
    end while
    -- Set the default sign.
    if sign = 0 then
        sign = 1
    end if
    
    for i = tstart to tend do
        v =  find(text[i],"0123456789.")
        -- Invalid char so force a zero return.
        if v = 0 then
            note = i
            if notify = 0 then
                sign = 0
            end if
            exit
        end if
        if v = 11 then
            if dot = 0 then
               dot = 1
            else
               note = i
                if notify = 0 then
                    sign = 0
                end if
               exit
            end if
         else
            if dot = 1 then
                rhs *= 10
                rh = (rh * 10) + v - 1
            else
                lhs += 1
                lh = (lh * 10) + v - 1
            end if
         end if

         -- I got to the end without error!
         if i = tend then
            note = 0
         end if
         
    end for

    if notify = 0 then
        return (lh + (rh / rhs)) * sign
    else
        return {(lh + (rh / rhs)) * sign, note}
    end if
end function

